@@ -1,21 +1,15 @@
+from contextlib import suppress
 from datetime import datetime
+from uuid import uuid4
 
 import aiohttp
-from contextlib import suppress
-
 import kopf
 import kubernetes_asyncio as kubernetes
 import pykube
-
-from uuid import uuid4
-
 from dask.compatibility import entry_points
-from distributed.core import rpc
-
 from dask_kubernetes.common.auth import ClusterAuth
-from dask_kubernetes.common.networking import (
-    get_scheduler_address,
-)
+from dask_kubernetes.common.networking import get_scheduler_address
+from distributed.core import rpc
 
 _ANNOTATION_NAMESPACES_TO_IGNORE = (
     "kopf.zalando.org",
@@ -327,6 +321,7 @@ async def retire_workers(
         scheduler_service_name,
         namespace,
         port_name="http-dashboard",
+        allow_external=False,
     )
     async with aiohttp.ClientSession() as session:
         url = f"{dashboard_address}/api/v1/retire_workers"
@@ -345,6 +340,7 @@ async def retire_workers(
         comm_address = await get_scheduler_address(
             scheduler_service_name,
             namespace,
+            allow_external=False,
         )
         async with rpc(comm_address) as scheduler_comm:
             return await scheduler_comm.workers_to_close(
@@ -371,6 +367,7 @@ async def get_desired_workers(scheduler_service_name, namespace, logger):
         scheduler_service_name,
         namespace,
         port_name="http-dashboard",
+        allow_external=False,
     )
     async with aiohttp.ClientSession() as session:
         url = f"{dashboard_address}/api/v1/adaptive_target"
@@ -385,6 +382,7 @@ async def get_desired_workers(scheduler_service_name, namespace, logger):
         comm_address = await get_scheduler_address(
             scheduler_service_name,
             namespace,
+            allow_external=False,
         )
         async with rpc(comm_address) as scheduler_comm:
             return await scheduler_comm.adaptive_target()
@@ -397,13 +395,25 @@ async def get_desired_workers(scheduler_service_name, namespace, logger):
 @kopf.on.update("daskworkergroup.kubernetes.dask.org")
 async def daskworkergroup_update(spec, name, namespace, logger, **kwargs):
     async with kubernetes.client.api_client.ApiClient() as api_client:
-        api = kubernetes.client.CoreV1Api(api_client)
+        customobjectsapi = kubernetes.client.CustomObjectsApi(api_client)
+        corev1api = kubernetes.client.CoreV1Api(api_client)
 
-        workers = await api.list_namespaced_pod(
+        cluster = await customobjectsapi.get_namespaced_custom_object(
+            group="kubernetes.dask.org",
+            version="v1",
+            plural="daskclusters",
+            namespace=namespace,
+            name=spec["cluster"],
+        )
+        cluster_labels = cluster.get("metadata", {}).get("labels", {})
+
+        workers = await corev1api.list_namespaced_pod(
             namespace=namespace,
             label_selector=f"dask.org/workergroup-name={name}",
         )
-        current_workers = len(workers.items)
+        current_workers = len(
+            [w for w in workers.items if w.status.phase != "Terminating"]
+        )
         desired_workers = spec["worker"]["replicas"]
         workers_needed = desired_workers - current_workers
         annotations = _get_dask_cluster_annotations(kwargs["meta"])
@@ -418,7 +428,8 @@ async def daskworkergroup_update(spec, name, namespace, logger, **kwargs):
                     annotations=annotations,
                 )
                 kopf.adopt(data)
-                await api.create_namespaced_pod(
+                kopf.label(data, labels=cluster_labels)
+                await corev1api.create_namespaced_pod(
                     namespace=namespace,
                     body=data,
                 )
@@ -435,7 +446,7 @@ async def daskworkergroup_update(spec, name, namespace, logger, **kwargs):
             )
             logger.info(f"Workers to close: {worker_ids}")
             for wid in worker_ids:
-                await api.delete_namespaced_pod(
+                await corev1api.delete_namespaced_pod(
                     name=wid,
                     namespace=namespace,
                 )
